@@ -4,12 +4,14 @@ import tqdm
 import gdown
 import torch
 import warnings
+import pyvirtualcam
 
 import numpy as np
 import torch.nn.functional as F
 import torchvision.transforms as transforms
 
 from PIL import Image
+from packaging import version
 
 filepath = os.path.abspath(__file__)
 repopath = os.path.split(filepath)[0]
@@ -20,15 +22,25 @@ from transparent_background.utils import *
 
 warnings.filterwarnings("ignore")
 
-URL = "https://drive.google.com/file/d/13oBl5MTVcWER3YU4fSxW3ATlVfueFQPY/view?usp=share_link"
-MD5 = "3bb068bd44574f0a0c39a8da900b1cf9"
+CONFIG = {
+'base': {'url': "https://drive.google.com/file/d/13oBl5MTVcWER3YU4fSxW3ATlVfueFQPY/view?usp=share_link",
+         'md5': "3bb068bd44574f0a0c39a8da900b1cf9",
+         'base_size': [1024, 1024],
+         'threshold': None,
+         'ckpt_name': "ckpt_base.pth",
+         'resize': dynamic_resize(L=1280)},
+'fast': {'url': "https://drive.google.com/file/d/1iRX-0MVbUjvAVns5MtVdng6CQlGOIo3m/view?usp=share_link",
+         'md5': "735a2fe8519bc12290f86bf7b8b395ff",
+         'base_size': [384, 384],
+         'threshold': 512,
+         'ckpt_name': "ckpt_fast.pth",
+         'resize': static_resize(size=[384, 384])}
+}
 
 class Remover:
-    def __init__(self, base_size=[1024, 1024], threshold=None, enable_pb=False, jit=False):
-        if jit & enable_pb:
-            raise AssertionError('jit and enable pyramid blending cannot be used together')
-        
-        self.model = InSPyReNet_SwinB(depth=64, pretrained=False, base_size=base_size, threshold=threshold)
+    def __init__(self, fast=False):
+        self.meta = CONFIG['fast' if fast else 'base']
+        self.model = InSPyReNet_SwinB(depth=64, pretrained=False, **self.meta)
         self.model.eval()
 
         checkpoint_dir = os.path.expanduser(os.path.join('~', '.transparent-background'))
@@ -36,26 +48,27 @@ class Remover:
             os.makedirs(checkpoint_dir, exist_ok=True)
     
         download = False
-        if not os.path.isfile(os.path.join(checkpoint_dir, 'latest.pth')):
+        ckpt_name = self.meta['ckpt_name']
+        
+        if not os.path.isfile(os.path.join(checkpoint_dir, ckpt_name)):
             download = True
-        elif MD5 != hashlib.md5(open(os.path.join(checkpoint_dir, 'latest.pth'), 'rb').read()).hexdigest():
+        elif self.meta['md5'] != hashlib.md5(open(os.path.join(checkpoint_dir, ckpt_name), 'rb').read()).hexdigest():
             download = True
         
         if download:
-            gdown.download(URL, os.path.join(checkpoint_dir, 'latest.pth'), fuzzy=True)
+            gdown.download(self.meta['url'], os.path.join(checkpoint_dir, ckpt_name), fuzzy=True)
 
-        self.model.load_state_dict(torch.load(os.path.join(checkpoint_dir, 'latest.pth'), map_location='cpu'), strict=True)
-        
-        self.use_gpu = False
+        self.model.load_state_dict(torch.load(os.path.join(checkpoint_dir, ckpt_name), map_location='cpu'), strict=True)
+
+        self.backend = "cpu"
         if torch.cuda.is_available():
-            self.model = self.model.cuda()
-            self.use_gpu = True
-            
-        if jit:
-            self.model = torch.jit.trace(self.model, torch.rand(1, 3, *base_size).cuda(), strict=False)
-            torch.jit.save(self.model, os.path.join(checkpoint_dir, 'jit.pt'))
-            
-        self.transform = transforms.Compose([dynamic_resize(L=1280),
+            self.backend = "cuda:0"
+        elif version.parse(torch.__version__) > version.parse("1.13") and torch.backends.mps.is_available():
+            self.backend = "mps:0"
+
+        self.model = self.model.to(self.backend)
+    
+        self.transform = transforms.Compose([self.meta['resize'],
                                             tonumpy(),
                                             normalize(mean=[0.485, 0.456, 0.406], 
                                                         std=[0.229, 0.224, 0.225]),
@@ -67,9 +80,7 @@ class Remover:
         shape = img.size[::-1]            
         x = self.transform(img)
         x = x.unsqueeze(0)
-        
-        if self.use_gpu:
-            x = x.cuda()
+        x = x.to(self.backend)
             
         with torch.no_grad():
             pred = self.model(x)
@@ -82,20 +93,25 @@ class Remover:
         
         if type == 'map':
             img = (np.stack([pred] * 3, axis=-1) * 255).astype(np.uint8)
+
         elif type == 'rgba':
             r, g, b = cv2.split(img)
             pred = (pred * 255).astype(np.uint8)
             img = cv2.merge([r, g, b, pred])
+
         elif type == 'green':
             bg = np.stack([np.ones_like(pred)] * 3, axis=-1) * [120, 255, 155]
             img = img * pred[..., np.newaxis] + bg * (1 - pred[..., np.newaxis])
+
         elif type == 'blur':
             img = img * pred[..., np.newaxis] + cv2.GaussianBlur(img, (0, 0), 15) * (1 - pred[..., np.newaxis])
+
         elif type == 'overlay':
             bg = (np.stack([np.ones_like(pred)] * 3, axis=-1) * [120, 255, 155] + img) // 2
             img = bg * pred[..., np.newaxis] + img * (1 - pred[..., np.newaxis])
             border = cv2.Canny(((pred > .5) * 255).astype(np.uint8), 50, 100)
             img[border != 0] = [120, 255, 155]
+
         elif type.lower().endswith(('.jpg', '.jpeg', '.png')):
             if self.background is None:
                 self.background = cv2.cvtColor(cv2.imread(type), cv2.COLOR_BGR2RGB)
@@ -106,9 +122,14 @@ class Remover:
 
 def console():
     args = parse_args()
-    remover = Remover(jit=args.jit)
+    remover = Remover(fast=args.fast)
 
-    if os.path.isdir(args.source):
+    if args.source.isnumeric() is True:
+        save_dir = None
+        _format = 'Webcam'
+        vcam = pyvirtualcam.Camera(width=640, height=480, fps=30)
+
+    elif os.path.isdir(args.source):
         save_dir = os.path.join(os.getcwd(), args.source.split(os.sep)[-1])
         _format = get_format(os.listdir(args.source))
 
@@ -119,7 +140,6 @@ def console():
     else:
         raise FileNotFoundError('File or directory {} is invalid'.format(args.source))
     
-
     if args.type == 'rgba' and _format == 'Video':
         raise AttributeError('type rgba cannot be applied to video input')
         
@@ -151,4 +171,8 @@ def console():
         if _format == 'Image':
             Image.fromarray(out).save(os.path.join(save_dir, '{}.png'.format(outname)))
         elif _format == 'Video' and writer is not None:
-            writer.write(cv2.cvtColor(out, cv2.COLOR_BGR2RGB))
\ No newline at end of file
+            writer.write(cv2.cvtColor(out, cv2.COLOR_BGR2RGB))
+        elif _format == 'Webcam':
+            # cv2.imshow('InSPyReNet', cv2.cvtColor(out, cv2.COLOR_BGR2RGB))
+            vcam.send(cv2.cvtColor(out, cv2.COLOR_BGR2RGB))
+            vcam.sleep_until_next_frame()
\ No newline at end of file
